@@ -1,4 +1,6 @@
 from copy import copy
+import inspect
+import itertools as it
 import os
 import warnings
 import sqlalchemy as sa
@@ -35,12 +37,14 @@ def log_sql(
     QueryPool.queries.append(statement)
 
 
-@mark.skipif("PY3 and os.environ.get('DB') == 'mysql'")
+# @mark.skipif("PY3 and os.environ.get('DB') == 'mysql'")
 class TestCase(object):
     versioning_strategy = 'subquery'
     transaction_column_name = 'transaction_id'
     end_transaction_column_name = 'end_transaction_id'
+    track_property_modifications = False
     store_data_at_delete = False
+    composite_pk = False
 
     @property
     def options(self):
@@ -49,6 +53,7 @@ class TestCase(object):
             'strategy': self.versioning_strategy,
             'transaction_column_name': self.transaction_column_name,
             'end_transaction_column_name': self.end_transaction_column_name,
+            'track_property_modifications': self.track_property_modifications,
             'store_data_at_delete': self.store_data_at_delete
         }
 
@@ -61,16 +66,14 @@ class TestCase(object):
         if adapter == 'postgres':
             dns = 'postgres://postgres@localhost/sqlalchemy_continuum_test'
         elif adapter == 'mysql':
-            dns = (
-                'mysql+pymysql://travis@localhost/sqlalchemy_continuum_test'
-                '?use_unicode=0&charset=utf8'
-            )
+            dns = 'mysql+pymysql://travis@localhost/sqlalchemy_continuum_test'
         elif adapter == 'sqlite':
             dns = 'sqlite:///:memory:'
         else:
             raise Exception('Unknown driver given: %r' % adapter)
 
         self.engine = create_engine(dns)
+        # self.engine.echo = True
         self.connection = self.engine.connect()
         self.Model = declarative_base()
 
@@ -128,3 +131,50 @@ class TestCase(object):
 
         self.Article = Article
         self.Tag = Tag
+
+
+setting_variants = {
+    'versioning_strategy': ['subquery', 'validity'],
+    'store_data_at_delete': [True, False],
+    'track_property_modifications': [True, False],
+    'transaction_column_name': ['transaction_id', 'tx_id'],
+    'end_transaction_column_name': ['end_transaction_id', 'end_tx_id']
+}
+
+
+def create_test_cases(base_class, setting_variants=setting_variants):
+    """
+    Function for creating bunch of test case classes for given base class
+    and setting variants. Number of test cases created is the number of linear
+    combinations with setting variants.
+
+    :param base_class:
+        Base test case class, should be in format 'xxxTestCase'
+    :param setting_variants:
+        A dictionary with keys as versioned configuration option keys and
+        values as list of possible option values.
+    """
+    names = sorted(setting_variants)
+    combinations = [
+        dict(zip(names, prod))
+        for prod in
+        it.product(*(setting_variants[name] for name in names))
+    ]
+
+    # Get the module where this function was called in.
+    frm = inspect.stack()[1]
+    module = inspect.getmodule(frm[0])
+
+    class_suffix = base_class.__name__[0:-len('TestCase')]
+    for index, combination in enumerate(combinations):
+        class_name = 'Test%s%i' % (class_suffix, index)
+        # Assign a new test case class for current module.
+        setattr(
+            module,
+            class_name,
+            type(
+                class_name,
+                (base_class, ),
+                combination
+            )
+        )
