@@ -1,7 +1,8 @@
-from tests import TestCase
+from six import PY3
+from tests import TestCase, create_test_cases
 
 
-class TestOneToManyRelationships(TestCase):
+class OneToManyRelationshipsTestCase(TestCase):
     def test_single_insert(self):
         article = self.Article()
         article.name = u'Some article'
@@ -21,10 +22,12 @@ class TestOneToManyRelationships(TestCase):
         self.session.commit()
         article.tags.remove(tag)
         self.session.commit()
+        assert article.versions.count() == 1
         article.tags.append(self.Tag(name=u'Some other tag'))
         article.name = u'Updated article'
         self.session.commit()
 
+        assert article.versions.count() == 2
         assert article.versions[0].tags.count() == 1
         assert article.versions[1].tags.count() == 1
         assert article.versions[1].tags[0].name == u'Some other tag'
@@ -56,3 +59,6 @@ class TestOneToManyRelationships(TestCase):
         self.session.commit()
         assert article.versions[0].tags.count() == 1
         assert article.versions[1].tags.count() == 0
+
+
+create_test_cases(OneToManyRelationshipsTestCase)
