@@ -1,4 +1,5 @@
 import sqlalchemy as sa
+from six import PY3
 from tests import TestCase
 
 
@@ -46,7 +47,7 @@ class TestUpdate(TestCase):
         article.name = u'Some article'
 
         self.session.commit()
-        assert article.versions.count() == 1
+        assert article.versions.count() == 2
 
     def test_stores_operation_type(self):
         article = self.Article()
@@ -87,7 +88,7 @@ class TestUpdateWithDefaultValues(TestCase):
 
             id = sa.Column(sa.Integer, autoincrement=True, primary_key=True)
             name = sa.Column(sa.Unicode(255))
-            updated_at = sa.Column(sa.DateTime, server_default=sa.func.now())
+            updated_at = sa.Column(sa.DateTime, default=sa.func.now())
             is_editable = sa.Column(sa.Boolean)
 
         self.Article = Article
